package no.roek.nlpgraphs.detailedanalysis;


import de.tudarmstadt.ukp.dkpro.lexsemresource.exception.LexicalSemanticResourceException;
import de.tudarmstadt.ukp.dkpro.lexsemresource.exception.ResourceLoaderException;
import no.roek.nlpgraphs.application.App;
import no.roek.nlpgraphs.application.PlagiarismSearch;
import no.roek.nlpgraphs.misc.ConfigService;
import no.roek.nlpgraphs.misc.DatabaseService;
import no.roek.nlpgraphs.misc.XMLUtils;

import java.util.Collections;
import java.io.IOException;
import java.util.List;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.concurrent.BlockingQueue;



public class PlagiarismWorker extends Thread {

	private BlockingQueue<PlagiarismJob> queue;
	public PlagiarismFinder plagFinder;
	private PlagiarismSearch concurrencyService;
	private String resultsDir;
    private boolean running;
	private int mergeDist;

	public PlagiarismWorker(BlockingQueue<PlagiarismJob> queue, PlagiarismSearch concurrencyService, DatabaseService db)
            throws LexicalSemanticResourceException, ResourceLoaderException, IOException {
		this.queue = queue;
		this.plagFinder = new PlagiarismFinder(db);
		this.concurrencyService = concurrencyService;
		ConfigService cs = App.getGlobalConfig();
        this.resultsDir = cs.getResultsDir();
		this.mergeDist = cs.getMergeDist();
	}


       public static <K extends Comparable,V extends Comparable> Map<K,V> sortByKeys(Map<K,V> map){
        List<K> keys = new LinkedList<K>(map.keySet());
        Collections.sort(keys);
      
        //LinkedHashMap will keep the keys in the order they are inserted
        //which is currently sorted on natural ordering
        Map<K,V> sortedMap = new LinkedHashMap<K,V>();
        for(K key: keys){
            sortedMap.put(key, map.get(key));
        }
      
        return sortedMap;
    }	
	

	@Override
	public void run() {
		running = true;
		while(running) {
			try {
				PlagiarismJob job = queue.take();
				if(job.isLastInQueue()) {
					running = false;
					break;
				}

			    Map<Integer,Integer> rowColumnMap = sortByKeys(plagFinder.findPlagiarism(job));                           
                          // List<PlagiarismReference> plagReferences= plagFinder.findPlagiarism(job);
                             List<PlagiarismReference> plagReferences= plagFinder.mergePassages(rowColumnMap);                                           
                           
			//   XMLUtils.writeResults(resultsDir, job.getSuspFile().getFileName().toString(), plagFinder.mergePassages(rowColumnMap));
                           XMLUtils.writeResults(resultsDir, job.getSuspFile().getFileName().toString(), PassageMerger.mergePassages(plagReferences,mergeDist));

				concurrencyService.plagJobDone(this, "queue: "+queue.size());
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}

	public synchronized void kill() {
		try {
			PlagiarismJob job = new PlagiarismJob("kill");
			job.setLastInQueue(true);
			queue.put(job);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
}
